/*
   EdgeRec.h

   EdgeRecs are edges with the additional data fields needed to keep track
   of scan conversion.  The AEL is a list of EdgeRecs, and the EdgeTable is
   full of them as well.
*/


/*
   EdgeRec will be used by the scan-converter
   to produce spans on each scanline.  Each EdgeRec points
   to the Edge and Polygon from which it arose.

   There are more, perhaps, data fields than strictly needed for any
   particular type of scan-conversion.
*/
class EdgeRec {
public:

  // Constructor 
  EdgeRec ();

  // see Scan.C for specification
  void Init (Edge *e0);

  // see Scan.C for specification
  void Update ();

  // print out a 2 line description of edge.
  void Print();

  // Compare EdgeRec a with b, on the x-coordinates.
  // -1 means a.x < b.x; 0 if a.x == b.x; 1 if a.x > b.x
  static int Compare ( EdgeRec *a, EdgeRec *b );


  //             The data members

  // Screen space ystart, and yend
  // ycurr has the current scanline
  float              ystart, yend, ycurr;

  // Screen space xstart, and xend
  // dx  = change in x/scanline
  float           xstart, xend, dx, xcurr;

  // Eye Space zstart, zend, zcurr and dz
  // Question: Should this be updated? Is it worth it?
  // Does z change linearly with y?
  // Does z change linearly with x?
  float           zstart, zend, dz, zcurr;

  // reciprocal of z
  // Does h change linearly with y?
  // Does h change linearly with x?
  float           hstart, hend, dh, hcurr; 

  // Optional Normal to be updated 
  Vector           nstart, nend, dn, ncurr;
  
  // Start and end color of the vertices of the edge
  Color            colstart, colend, dcol, colcurr;

  // The original edge from which this EdgeRec was built
  Edge             *edge;

  // The polygon from which this Edge (and EdgeRec) arose
  Polygon          *poly;
};


