/*
  List.C is the doubly-linked list template class, with a current position maintained
  in it.

  You should not need to modify this file.  See List.h for specifics on various methods.
  Some details are in List.C, below, as well.
*/

#include <stdio.h>
#include <stdlib.h>

/*
  this is worth looking at.
*/
#include "List.h"


template<class LType> boolean CLinkList<LType> :: Empty()
{
	return (begin == NULL && end == NULL);
}

template<class LType> boolean CLinkList<LType> :: IsEnd()
{
	return (current == end || end == NULL);
}

template<class LType> boolean CLinkList<LType> :: IsFront()
{
  return (current == begin || begin == NULL);
}

template<class LType> void CLinkList<LType> :: Add(LType *temp)
{
  assert (temp != NULL);
	CObject<LType> *last;
	last = end;
	end = new CObject<LType>(temp, last, NULL);
	if(last!=NULL)
		last->SetNext(end);
	else
		current = begin = end;
}

template<class LType> LType * CLinkList<LType> :: Prev()
{
	if(current!=begin)
	  {
	    current = current->GetPrior();
	    return current->GetObject();
	  }
	else 
	  return NULL;
}

template<class LType> LType * CLinkList<LType> :: Next()
{
	if(current!=end)
	  {
	    current = current->GetNext();
	    return (current == NULL) ? NULL : current->GetObject();
	  }
	else
	  return NULL;
}


template<class LType> LType * CLinkList<LType> :: Insert(LType *temp)
{
  assert (temp != NULL);
	if(current == NULL)
		Add(temp);
	else
	{
		current->SetNext(new CObject<LType>(temp, current,
						    current->GetNext()));
		if(current == end)
			end = current->GetNext();
		current = current->GetNext();
	}
	return  current->GetObject();

}



// Effect:  Insert before the current node.  Remain on current node.
template<class LType> LType * CLinkList<LType> :: InsertBefore(LType *temp)
{
  assert (temp != NULL);
  if(current == NULL)
    Add(temp);
  else
    {
      current->SetPrev(new CObject<LType>(temp,
					  current->GetPrev(), 
					  current ) );
      if(current == begin)
	begin = current->GetPrev();
    }
  return current;
}

// Effect: remove current element; return next element or NULL if none
template<class LType> LType * CLinkList<LType> :: Remove()
{
  // can't remove from empty list
  assert ( current != NULL );

  if(current == begin && current == end) // one element on list, remove it.
    {
      delete current;
      begin = end = current = NULL;
      return NULL;
    }
  else if(current == begin) // remove first element, advance
    {
      begin = current->GetNext();
      delete current;
      current = begin;
      begin->SetPrior(NULL);
      return current->GetObject();  
    }
  else if(current == end)  // remove last element, advance
    {
      end = current->GetPrior();
      end->SetNext(NULL);
      delete current;
      current = end;
      return NULL; // no next element
    }
  else
    {
      CObject<LType> *temp;
      temp = current->GetNext();
      temp->SetPrior(current->GetPrior());
      current->GetPrior()->SetNext(temp);
      delete current;
      current = temp;
      return current->GetObject();  
    }
}

template<class LType> void CLinkList<LType> :: Clear()
{
	CObject<LType> *destroy;

	ToFront();
	while(!IsEnd())
	{
		destroy = current;
		Next();
		delete destroy;
	}
	delete current;
	begin = end = current = NULL;
}


// Effect: Concats b onto current list.
//         Empties out b. 
// Modifies: potentially changes head.
// Requires: 
template<class LType> void CLinkList<LType> :: Concat (CLinkList<LType> *b)
{
  assert(b != NULL);

  if (b->Empty())
    {
      return;
    }

  if (Empty()) 
    { 
      // am empty, just move b onto me.
      begin = b->begin;
      end = b->end;
      current = b->begin;
    }

  else {
    // am not empty.  Add b's to end of me.
    assert(end != NULL);
    end->SetNext(b->begin);

    assert(b->begin != NULL);
    b->begin->SetPrior(end);

    assert(b->end != NULL);
    end = b->end;
  }

  // empty out b without freeing elements in it (since elements are now
  // in a)
  b->begin = b->end = b->current = NULL;
}




