/*Code by Jason Yang */
/* templated doubly linked list with current pointer */
/* modified by 6.837 staff */
/* Note:  you should not modify this file. */

#ifndef LINKLIST_H
#define LINKLIST_H

#include <iostream.h>
#include "Types.h"

//  This is a node in the doubly-linked list below.
//  It has pointers to the next node (in the list), previous node, and its
//  satellite data.
template <class OType> class CObject
{
        public:
                CObject(OType *o, CObject*p, CObject*n) 
		  {obj = o; prior = p; 
		  next = n;};
                virtual ~CObject() {};
                
                CObject *GetPrior()        {return prior;};
                CObject *GetNext()         {return next;};
                OType *GetObject()         {return obj;};
                void SetPrior(CObject *temp) {prior = temp;};
                void SetNext(CObject *temp)  {next = temp;};


        private:
                OType *obj;
                CObject *prior;
                CObject *next;
};


// Implement a doubly-linked list.
// Maintains beginning, end, and a current position.
// The list can be used like a tape:  it can be advanced in either direction,
//   rewound, etc.
// The variable current serves as the tape head, pointing to the element
//   at the current position on the list.
// You can splice elements out of the list, concatenate two lists,
//   empty a list, etc.
// Function headers are in List.C.
template<class LType> class CLinkList
{
	public:
                // make empty list
		CLinkList(void) {begin = end = current = NULL;};

                // make single element into list
		CLinkList(LType *) 
		  {begin = end = current = new
		     CObject<LType>(temp, NULL, NULL);};

                // destructor (frees memory)
		virtual ~CLinkList(void) {Clear();};

                // append element to list
		void Add(LType *);

                // insert element directly after current
		LType *Insert(LType *);

                // insert element directly before current
		LType *InsertBefore(LType *);

                // splice current out of list (i.e. remove current)
                // and set current to the following element
                // (if there is no following element, current gets
                //  set to the last element).
                //  returns the new value of current in the first
                //  case and null if there was no following element.
		LType *Remove();

                // go back one element
                // returns new value of current or null if already were
                // at beginning
		LType *Prev();

                // go forward one element
                // returns new value of current or null if already were
                // at end
		LType *Next();

                // return the current element
		LType *GetCurrent() 
		  {return (current == NULL) ? NULL : current->GetObject();}

                // rewind to the beginning (set current to first element)
		LType *ToFront() {current = begin; return GetCurrent(); }

                // advance to the end (set current to last element)
		LType *ToEnd() {current = end; return GetCurrent(); }

                // clear list (make it have no elements)
                // elements are removed from list but not destroyed
		void Clear();

                // return true iff at beginning of list
		boolean IsFront();

                // return true iff at end of list
		boolean IsEnd();

                // return true iff list is empty (has no elements)
		boolean Empty();

                // concatenates two lists, with this list coming first.
                // transfers elements from argument list to end of
                // this list, preserving their order.
		void Concat(CLinkList<LType> *);

      protected:
		CObject<LType> *begin; // beginning of list
		CObject<LType> *end; // end of list
		CObject<LType> *current; // marks the current element
                                         // always points to some element
                                         // if list is non-empty
		
};

#endif



