/*
   Main.C -- main() routine of ivscan 

   I would not modify this file.
*/

// system includes
#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <iostream.h>

// X includes
#include <X11/Intrinsic.h>

// GL includes
#include <GL/gl.h>
#include <GL/GLwDrawA.h>

// Inventor includes
#include <Inventor/SoDB.h>
#include <Inventor/SoInteraction.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/nodekits/SoBaseKit.h>
#include <Inventor/nodekits/SoNodeKit.h>
#include <Inventor/nodes/SoSelection.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/sensors/SoFieldSensor.h>
#include <Inventor/sensors/SoTimerSensor.h>

// local includes
#include "ivwraplib/SoSceneViewer.h"
#include "Interface.h"


// forward decs
void ButtonCB (Widget, XtPointer data, XtPointer);
void ButtonCBD0 (Widget, XtPointer data, XtPointer);
void ButtonCBD1 (Widget, XtPointer data, XtPointer);
void ButtonCBD2 (Widget, XtPointer data, XtPointer);
void ButtonCBD4 (Widget, XtPointer data, XtPointer);
void ButtonCBD8 (Widget, XtPointer data, XtPointer);
void ButtonCBD16 (Widget, XtPointer data, XtPointer);
void ButtonCBD32 (Widget, XtPointer data, XtPointer);


// main procedure
void main (int argc, char **argv)
{ 
  SoInput in;

  // parse arguments, complain if bad usage
  if (!getArgs(argc, argv)) Usage(argv);

  // workaround for bug 200909 - this will force OpenGL to create
  // a connection with the X server to receive delete window events
  // (so that OpenGL can delete Accumulation and other software
  // buffers when the window is destroyed).
  if (putenv("GL_CHECK_WINDOW_DESTROY=y"))
    fprintf(stderr, "Couldn't set GL_CHECK_WINDOW_DESTROY env.\n");

  // init Inventor
  SoDB::init();

  // create selection node as scene graph root
  SoSelection *selRoot = new SoSelection;
  selRoot->ref();

  // read inventor file
  SbBool doRead = FALSE;

  // if filename is "-"; read from stdin
  if (interface::filename && !strcmp (interface::filename, "-")) {
    doRead = TRUE;
    in.setFilePointer (stdin);
  } else if (interface::filename) 
    doRead = in.openFile(interface::filename);

  // otherwise read from supplied file
  if (doRead) {
    fprintf(stderr, "Reading input file... ");
    SoNode *n;
    while ((SoDB::read(&in, n) != FALSE) && (n != NULL))
      selRoot->addChild(n);
    fprintf(stderr, selRoot->getNumChildren() ?
	    "done.\n" : "No data read; creating empty scene.\n");
  }

  // initialize inventor window
  Widget mainWindow = SoXt::init(argv[0]);
      
  // install scan converter button and callback (debug 0)
  installSVbuttonCB("S", "ScanConvert", ButtonCBD0);

  // install scan converter button and callback (debug 1)
  installSVbuttonCB("D", "ScanConvert", ButtonCBD1);

  // install scan converter button and callback (debug 2)
  installSVbuttonCB("B", "ScanConvert", ButtonCBD2);

  // install scan converter button and callback (blowup 4)
  installSVbuttonCB("4", "ScanConvert", ButtonCBD4);

  // install scan converter button and callback (blowup 8)
  installSVbuttonCB("8", "ScanConvert", ButtonCBD8);

  // install scan converter button and callback (blowup 16)
  installSVbuttonCB("16", "ScanConvert", ButtonCBD16);

  // install scan converter button and callback (blowup 32)
  installSVbuttonCB("32", "ScanConvert", ButtonCBD32);

  // create (modified) SceneViewer
  SoSceneViewer *sv = new SoSceneViewer (mainWindow, NULL, 
					 TRUE, selRoot, interface::envname);
  selRoot->unref();
      
  // realize (modified) SceneViewer
  sv->show();
  XtRealizeWidget(mainWindow);
      
  // loop until done
  SoXt::mainLoop();
} // end main


// This function is called whenever a "S", "D", or any other buttion on the
// left hand side of the application is pressed 
// Effect: Calls ScanScene() and displays result.
void ButtonCB (Widget, XtPointer data, XtPointer)
{
  SoSceneViewer *sViewer = (SoSceneViewer *) data;

  // Create environment.  The viewvolume, viewport, etc. 
  // will be setup in env, and env will be passed to the
  // scan conversion routines. 
  Environment* env = new Environment;  
  assert (env);

  // make sure camera is perspective (else eye is at infinity)
  assert(sViewer->getCamera());
  if (!sViewer->getCamera()->isOfType(SoPerspectiveCamera::getClassTypeId())) {
    cerr << "Error: Only the perspective camera is supported."
	 << endl
	 << "Use the frustum icon to switch the camera type." << endl;
    return;
  }

  // create view volume from the current camera.
  SoPerspectiveCamera *cam = (SoPerspectiveCamera *) sViewer->getCamera();
  env->viewVolume.eye = cam->position.getValue();
  assert(sViewer->getCurrentViewer());
  SbViewportRegion region = sViewer->getCurrentViewer()->getViewportRegion();
  env->avpSize = env->wvpSize = region.getViewportSizePixels();
  if ( interface::blowup ) { // "blow up" pixels to NxN
    env->wvpSize[0] = env->avpSize[0] / interface::blowup;
    env->wvpSize[1] = env->avpSize[1] / interface::blowup;
    }

  // see SoCamera (OIC++RM p. 133) for documentation
  // set up the env viewVolume
  cam->orientation.getValue().multVec(SbVec3f(0,0,-1), env->viewVolume.viewDir);
  cam->orientation.getValue().multVec(SbVec3f(0,1,0), env->viewVolume.upDir);

  env->viewVolume.aspectRatio = env->avpSize[0] / (float) env->avpSize[1];

  env->viewVolume.nearDistance = cam->nearDistance.getValue();
  env->viewVolume.farDistance = cam->farDistance.getValue();
  env->viewVolume.fovy = cam->heightAngle.getValue();
  env->viewVolume.orientation = cam->orientation.getValue();

  // set up the env scene graph
  env->sceneGraph = sViewer->getSceneGraph();
  assert (env->sceneGraph);

  // draw in normal window
  GLwDrawingAreaMakeCurrent (sViewer->getNormalWidget (),
			     sViewer->getNormalContext ());

  // save openGL state
  GLint mMode, dBuffer;
  GLfloat clearColor[4];
  glGetIntegerv (GL_MATRIX_MODE, &mMode);
  glGetIntegerv (GL_DRAW_BUFFER, &dBuffer);
  glGetFloatv  (GL_COLOR_CLEAR_VALUE, clearColor);
  glClearColor(0, 0, 0, 1);
  glClear (GL_DEPTH_BUFFER_BIT);

  //  show pixel writes to front buffer
  glDrawBuffer(GL_FRONT);

  // set up screen coordinates
  glOrtho (-0.5f, env->avpSize[0] - 0.5f, -0.5f, env->avpSize[1] - 0.5f, -1, 1);

  // if "blank" button pushed, clear
  if ( interface::debug == 2 )
    glClear (GL_COLOR_BUFFER_BIT);

  // call user routine ScanScene() from Interface.C
  ScanScene (env);

  // force the graphics pipe to finish
  glFlush();

  // Restore saved GL state
  glClearColor (clearColor[0], clearColor[1], clearColor[2], clearColor[3]);
  glMatrixMode (mMode);
  glDrawBuffer (dBuffer);

}

void ButtonCBD0 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 0;
  interface::blowup = 0;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD1 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 1;
  interface::blowup = 0;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD2 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 2;
  interface::blowup = 0;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD4 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 0;
  interface::blowup = 4;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD8 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 0;
  interface::blowup = 8;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD16 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 0;
  interface::blowup = 16;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}

void ButtonCBD32 (Widget, XtPointer data, XtPointer)
{
  int sd = interface::debug;
  interface::debug = 0;
  interface::blowup = 32;
  ButtonCB (0, data, 0);
  interface::debug = sd;
}










