/*
   Polygon.h

   The Polygon class and some associated structures and typedefs.
*/

#ifndef __POLYGON_H__
#define __POLYGON_H__

// Here we hard-code the number of verticies found in a polygon
#define POLYNUMVERTS  3
#define POLYNUMEDGES  3

// PolyVert represents a vertex in a polygon.
// It has the Screen Space, Eye Space coordinates, Eye Space normal
// and the color of the vertex.
struct PolyVert {
    Point2D     sspt;
    Point       espt;
    Color       col;
    Vector      norm;
};    

// In this problem set each polygon has only 3 vertices
// PolyVert is the vertex data structure
class Polygon {
public:
  // Constructor
  Polygon () {
    for (int i = 0; i < POLYNUMVERTS; i++) p[i] = PolyVert (); 
    in = FALSE;
  }

  // see ScanWrap.C for specification
  void SortPts ();
  
  // Initialize p[0..2]
  void Init (Point *w, Point2D *s, Color *c, Vector *n) {
    for (int i = 0; i < POLYNUMVERTS; i++) {
      p[i].espt = w[i];
      p[i].sspt = s[i];
      p[i].col  = c[i];
      p[i].norm = n[i];
    }
  }

  // Return vertex <id>
  PolyVert pt (int id) {return p[id];}

  // TRUE iff scan converter is building a span for this polygon
  Binary      in;

private:
  PolyVert    p[POLYNUMVERTS];
};


#endif
