// utils.h: math utility functions

#ifndef _SC_UTILS_
#define _SC_UTILS_

#include <math.h>
#include <stdlib.h>	// for rand()
#include <assert.h>
#include "SCtypes.h"

// utility functions

// min two floats
float fmin(float a, float b);

// max two floats
float fmax(float a, float b);

// return random float in [low..high]
float randr(float low, float high);

// max component of reflectance
float vmax(Color color);

// multiply two vectors, componentwise, to produce vector
Vector vmult(Vector &a, Vector &b);

// perpendicular point/plane distance
float distance(Plane &H, Point &p);

// adjust H.d to contain p
int plane_from_three_points(Point &a, Point &b, Point &c, Plane &H);

// construct plane from point p and normal n
void plane_from_point_normal(Point &p, Vector &n, Plane &H);

// construct plane containing p, parallel to par, perpendicular to perp
int plane_from_point_perp_par(Point &p, Vector &par, 
			       Vector &perp, OUTPUT Plane &H);

// save a block of pixels to a file
void saveWindow(unsigned long *pixels, int width, int height, int inspect);

// put a pixel on the screen now
void setPixelImmed(int x, int y, SbColor color, Environment *env);

// put a 4x4 pixel on the screen now
void setPixel4(int x, int y, SbColor color, Environment *env);

// put an 8x8 pixel on the screen now
void setPixel8(int x, int y, SbColor color, Environment *env);

// put a 16x16 pixel on the screen now
void setPixel16(int x, int y, SbColor color, Environment *env);

// put a 32x32 pixel on the screen now
void setPixel32(int x, int y, SbColor color, Environment *env);

// put a pixel on the screen later
void setPixelDelayed(int x, int y, SbColor color, Environment *env);

// put a pixel in the buffer without the screen
void setPixelBuffer(int x, int y, SbColor color, Environment *env);

Color CompMult (Color c1, Color c2);

void drawLine (int x1, int y1, int x2, int y2, 
	       SbColor color, Environment *env);

#endif /* !_SC_UTILS_ */








